import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Window
import QtQuick.VirtualKeyboard.Settings

import Core
import Sco

import "widgets"

/**
 * @brief Основное окно
 */
ApplicationWindow {
    id: root
    objectName: "qmlMainWindow"
    width: Style.parameter("width")
    height: Style.parameter("height")
    visible: true

    function topPanelColor(c: color) {
        // Один из способов проверить что цвет валидный https://bugreports.qt.io/browse/QTBUG-78325
        return Qt.colorEqual(Qt.rgba(c.r, c.g, c.b, c.a), c) ? c : Style.color("background")
    }

    ColumnLayout {
        anchors.fill: parent
        spacing: 0

        TopPanel {
            id: topPanel
            z: 1
            visible: !workArea.currentItem || !workArea.currentItem.Context.fullScreen || dialogWindow.visible

            property color baseColor
            property color targetColor
            property real targetOpacity: workArea.currentItem ? workArea.currentItem.opacity : 1
            backgroundColor: Qt.tint(baseColor, Qt.rgba(targetColor.r, targetColor.g, targetColor.b, targetOpacity))

            onLogo: () => mainWindow.logo()

            Layout.fillWidth: true
        }

        StackView {
            id: workArea

            pushEnter: null
            pushExit: null
            popEnter: null
            popExit: null
            replaceEnter: null
            replaceExit: null

            Layout.fillWidth: true
            Layout.fillHeight: true

            function removeItem(item: Item) {
                if (this.get(item.StackView.index) !== item)
                    return

                this.pushEnter = null
                this.pushExit = null
                this.popEnter = null
                this.popExit = null

                const tmp = []
                while (tmp[tmp.length - 1] !== item && workArea.depth > 1) {
                    const popped = this.pop(StackView.Immediate)
                    tmp.push(popped)
                }

                if (tmp[tmp.length - 1] === item)
                    tmp.pop()

                if (tmp.length > 0) {
                    this.popEnter = tmp[tmp.length - 1].popEnter
                    this.push(tmp, StackView.PopTransition)
                } else if (this.currentItem) {
                    this.popEnter = workArea.currentItem.popEnter
                    this.replace(workArea.currentItem, workArea.currentItem, StackView.PopTransition)
                }
            }
        }
    }

    Dialog {
        id: modalBackground
        anchors.centerIn: Overlay.overlay
        closePolicy: Popup.NoAutoClose
        modal: true
        z: 3; width: 0; height: 0

        Overlay.modal: Rectangle {
            color: Style.color("modalBackground")
        }

        // Дублируем лого, так как на оригинальный объект нажать теперь нельзя
        MouseArea {
            parent: modalBackground.contentItem
            z: 5
            property var pos: parent ? parent.mapFromItem(Overlay.overlay, topPanel.leftPadding, topPanel.topPadding) : Qt.point(0, 0)
            x: pos.x; y: pos.y
            height: topPanel.appLogo.height
            width: topPanel.appLogo.width

            onClicked: () => topPanel.logo()
        }
    }

    Window {
        id: dialogWindow
        color: "transparent"
        flags: Qt.FramelessWindowHint

        transientParent: root

        property string lastGeometry

        function updateGeometry() {
            x = root.x + (root.width - dialogBox.implicitWidth) / 2
            y = root.y + (root.height - dialogBox.implicitHeight) / 2
            width = dialogBox.implicitWidth
            height = dialogBox.implicitHeight
            const geometry = width + "x" + height
            if (geometry !== lastGeometry) {
                lastGeometry = geometry
                Utils.applyWindowRadius(this, dialogBox.Style.border.radius)
            }
        }

        GroupBox {
            id: dialogBox
            Style.class: "modal"
            anchors.fill: parent
            padding: 40
            topPadding: 96
            clip: true

            onImplicitHeightChanged: () => dialogWindow.updateGeometry()

            MouseArea {
                x: 0; y: 0
                width: 100; height: 100

                onClicked: () => topPanel.logo()
            }
        }
    }

    Window {
        id: keyboardWindow
        flags: Qt.FramelessWindowHint | Qt.WindowStaysOnTopHint | Qt.Popup | Qt.NoFocus
        x: root.x + (root.width - width) / 2

        width: 1010
        height: 810

        property real radius: keyboardContainer.Style.border.radius

        Component.onCompleted: () => Utils.applyWindowRadius(this, radius)
        onRadiusChanged: () => Utils.applyWindowRadius(this, radius)

        function updateFocusedItem() {
            let newFocusedItem
            if (Utils.isQmlWindow()) {
                newFocusedItem = dialogWindow.visible ? dialogWindow.activeFocusItem : root.activeFocusItem
            } else {
                newFocusedItem = Utils.focusWidget()
                if (focusedItem !== void 0 && focusedItem.showKeyboard !== true)
                    return
            }
            if (focusedItem !== newFocusedItem)
                resetFocusedItem()
            if (focusedItem !== void 0 && focusedItem.text !== void 0) {
                if (typeof focusedItem.displayTextChanged === 'function') {
                    focusedItemConnectionDisplayTextChanged = function() { keyboard.text = focusedItem.displayText }
                    focusedItem.displayTextChanged.connect(focusedItemConnectionDisplayTextChanged)
                    keyboard.text = focusedItem.displayText
                }
                if (typeof focusedItem.focusChanged === 'function') {
                    focusedItemConnectionDisplayTextChanged = function() { updateFocusedItem() }
                    focusedItem.focusChanged.connect(focusedItemConnectionFocusChanged)
                }
                if (typeof focusedItem.selectText === 'function')
                    focusedItem.selectText()
            } else {
                keyboard.text = ""
            }

            showKeyboard()
        }

        function resetFocusedItem() {
            if (focusedItem !== void 0) {
                if (typeof focusedItem.clearFocus === 'function')
                    focusedItem.clearFocus()
                if (focusedItemConnectionDisplayTextChanged !== void 0)
                    focusedItem.displayTextChanged.disconnect(focusedItemConnectionDisplayTextChanged)
                if (focusedItemConnectionFocusChanged !== void 0)
                    focusedItem.focusChanged.disconnect(focusedItemConnectionFocusChanged)
                if (typeof focusedItem.clearFocus === 'function')
                    focusedItem.clearFocus()
                focusedItem = void 0
                focusedItemConnectionDisplayTextChanged = void 0
                focusedItemConnectionFocusChanged = void 0
            }
        }

        function showKeyboard() {
            keyboardContainer.state = "show"
            keyboardWindow.show()
            keyboardWindow.raise()
        }

        function hideKeyboard() {
            resetFocusedItem()
            keyboardContainer.state = "hide"
            Qt.callLater(Qt.inputMethod.hide)
        }

        function confirm() {
            Qt.inputMethod.commit()
            hideKeyboard()
        }

        function reject() {
            if (focusedItem !== void 0 && focusedItem.text !== void 0) {
                const text = focusedItem.text
                Qt.inputMethod.commit()
                focusedItem.text = text
            }

            hideKeyboard()
        }

        property bool enabled: allowFloatKeyboard && Qt.inputMethod.visible
        property var focusedItem
        property var focusedItemConnectionDisplayTextChanged
        property var focusedItemConnectionFocusChanged
        property bool allowFloatKeyboard: false
        onEnabledChanged: enabled ? updateFocusedItem() : resetFocusedItem()

        GroupBox {
            id: keyboardContainer
            anchors.fill: parent
            Style.class: "fullSizedKeyboardWindow"
            FullSizeKeyboard {
                id: keyboard

                anchors.fill: parent
                onConfirm: keyboardWindow.confirm()
                onCancel: keyboardWindow.reject()
            }
            state: "hide"
            states: [
                State { name: "hide"; PropertyChanges { target: keyboardWindow; y: root.height } },
                State { name: "show"; PropertyChanges {
                        target: keyboardWindow; y: root.height - height - (root.width - width) / 2
                    }
                }
            ]
            transitions: [
                Transition { from: "hide"; to: "show"
                    SmoothedAnimation {
                        velocity: Style.animationVelocity("transition.keyboard", 2000)
                        property: "y"
                    }
                },
                Transition { from: "show"; to: "hide"
                    SequentialAnimation {
                        SmoothedAnimation {
                            velocity: Style.animationVelocity("transition.keyboard", 5000)
                            property: "y"
                        }
                        ScriptAction { script: keyboardWindow.hide() }
                    }
                }
            ]
        }
    }

    Component.onCompleted: () => {
        let langs = Config.getList("I18n:keyboardLayouts", ["ru", "en"])
        let locales = []
        for (let i = 0; i < langs.length; i++) {
            switch(langs[i]) {
                case "ru": locales.push("ru_RU"); break;
                case "en": locales.push("en_US"); break;
            }
        }
        VirtualKeyboardSettings.activeLocales = locales;
        requestActivate()
    }

    Connections {
        target: mainWindow

        function onShowMainContext(context, item) {
            if (item) {
                if (!Sco.startup) {
                    workArea.pushEnter = item.pushEnter
                    workArea.pushExit = null
                    workArea.popEnter = item.popEnter
                    workArea.popExit = null
                }

                if (workArea.get(item.StackView.index) !== item) {
                    if (workArea.currentItem) {
                        workArea.pushExit = workArea.currentItem.pushExit
                        topPanel.baseColor = topPanelColor(workArea.currentItem.Context.topPanelColor)
                    }

                    workArea.push(item)
                    context.removed.connect(() => workArea.removeItem(item))
                } else {
                    workArea.pop(item)
                }

                topPanel.targetColor = topPanelColor(item.Context.topPanelColor)

                root.show()
                root.raise()
                item.focus = true
            } else {
                root.hide()
            }

            keyboardWindow.hideKeyboard(true)
        }

        function onShowModalContext(context, item) {
            if (dialogWindow.visible && dialogBox.contentItem)  {
                dialogWindow.hide()
                dialogBox.contentItem = null
            }

            if (item) {
                keyboardWindow.allowFloatKeyboard = item.allowFloatKeyboard
                dialogBox.contentItem = item
                item.visible = true
                dialogWindow.updateGeometry()
                dialogWindow.show()
                dialogWindow.requestActivate()
                Qt.callLater(() =>  {
                    Qt.callLater(() => {
                        dialogWindow.raise()
                        item.focus = true
                        if (typeof item.activated === 'function')
                            item.activated()
                    });
                })
            } else if (root.visible && workArea.currentItem) {
                keyboardWindow.allowFloatKeyboard = workArea.currentItem.allowFloatKeyboard
                if (typeof workArea.currentItem.activated === 'function')
                    workArea.currentItem.activated()
            } else {
                keyboardWindow.allowFloatKeyboard = true
            }
        }

        function onModalChanged(modal) {
            modalBackground.visible = modal
        }
    }
}
