import QtQuick
import QtQuick.Controls
import QtQuick.Layouts

import Core
import Check
import I18n

import "../core"
import "../widgets"

ModalForm {
    id: root
    Style.class: "inputCardForm"

    property int currentIndex: -1
    property var currentMethod: model.get(currentIndex)

    Context.inputSources: InputSources.Msr | InputSources.Scanner

    Component.onCompleted: () => {
        const methods = Config.getList("Check:cardInputMethods", ["read"])
                // Удаляем дубликаты
                .filter((item, pos, self) => self.indexOf(item) === pos)

        for (const m of methods) {
            switch (m) {
            case "phone":
                model.append({ button: "inputCardByPhone", cardInputMethod: CheckTypes.CardInputMethod.Phone, manual: true })
                break
            case "number":
                model.append({ button: "inputCardByNumber", cardInputMethod: CheckTypes.CardInputMethod.Number, manual: true })
                break
            case "read":
                model.append({ button: "inputCardByReading", cardInputMethod: CheckTypes.CardInputMethod.Number,
                        hint: "inputCardByReadingHint", manual: false })
                break
            case "paymentCode":
                model.append({ button: "inputCardByPaymentCode", cardInputMethod: CheckTypes.CardInputMethod.PaymentCode, manual: true })
                break
            }
        }

        if (model.count == 0)
            model.append({ button: "inputCardByReading", cardInputMethod: CheckTypes.CardInputMethod.Number,
                    hint: "inputCardByReadingHint", manual: false })

        currentIndex = 0
    }

    ListModel {
        id: model
    }

    // При передаче параметра cardInputMethod через QVariantMap в PluginManager.async происходят проблемы
    Action {
        id: addCardAction
        command: "CHECK_ADDCARD"

        property string number
        property int source: EInput.Keyboard
        property int cardInputMethod
    }

    function accept() {
        if (!inputField.acceptableInput)
            return

        if (currentMethod.cardInputMethod == CheckTypes.CardInputMethod.Phone)
            addCardAction.number = inputField.text.replace(/\(|\)|-/g, '')
        else
            addCardAction.number = inputField.text
        addCardAction.cardInputMethod = currentMethod.cardInputMethod
        addCardAction.exec()
    }

    function reject() {
        removeContext()
    }

    Keys.onEnterPressed: () => accept()
    Keys.onReturnPressed: () => accept()
    Keys.onEscapePressed: () => reject()

    onCurrentMethodChanged: () => {
        inputField.text = ''
        if (currentMethod)
            Check.cardInputMethod = currentMethod.cardInputMethod
    }

    contentItem: ColumnLayout {
        spacing: 40

        Label {
            id: title
            Style.class: "inputCardTitle"
            horizontalAlignment: Label.AlignHCenter
            wrapMode: Label.WordWrap

            Tr { code: "inputCardTitle" }

            Layout.fillWidth: true
        }

        Label {
            id: message
            visible: model.count > 1
            Style.class: "inputCardMessage"
            horizontalAlignment: Label.AlignHCenter
            wrapMode: Label.WordWrap

            Tr { code: "inputCardMessage"; fallback: "" }

            Layout.fillWidth: true
        }

        Item {
            id: buttonsLayout
            property real spacing: 10
            property bool modelOdd: (model.count % 2) != 0
            property int rows: (model.count + modelOdd) / 2

            visible: model.count > 1
            width: root.width
            height: rows * 71 + (rows - 1) * spacing

            Repeater {
                model: model
                delegate: ActionButton {
                    Style.class: "inputCardMethod"
                    property bool odd: (model.index % 2) != 0
                    property int row: ((model.index + buttonsLayout.modelOdd) / 2)
                    property int col: buttonsLayout.modelOdd ? ((model.index + !!model.index) % 2) : (model.index % 2)

                    width: buttonsLayout.modelOdd && model.index == 0 ? buttonsLayout.width : (buttonsLayout.width - buttonsLayout.spacing) / 2
                    height: 71
                    x: col == 0 ? 0 : (width + buttonsLayout.spacing)
                    y: row * (height + buttonsLayout.spacing)
                    enabled: root.currentIndex != model.index
                    Tr { code: model.button }

                    onClicked: () => currentIndex = model.index
                }
            }
        }

        Label {
            id: hint
            Style.class: "inputCardHint"
            visible: currentMethod.hint !== void 0 && currentMethod.hint.length > 0
            horizontalAlignment: Qt.AlignHCenter
            wrapMode: Label.WordWrap

            Tr { code: currentMethod.hint ? currentMethod.hint : "" }

            Layout.fillWidth: true
        }

        SvgImage {
            visible: !currentMethod.manual
            source: Style.parameter("hint.inputCard")

            Layout.leftMargin: 90
            Layout.topMargin: 80
            Layout.rightMargin: 90
            Layout.bottomMargin: 45
            Layout.alignment: Qt.AlignHCenter
        }

        ColumnLayout {
            spacing: 40
            visible: currentMethod.manual

            GroupBox {
                Style.class: "inputCardInputField"
                contentItem: FocusScope {
                    focus: true

                    implicitHeight: inputField.implicitHeight

                    TextEdit {
                        id: inputField
                        anchors.fill: parent

                        inputMethodHints: Qt.ImhDigitsOnly
                        focus: true
                        horizontalAlignment: Qt.AlignHCenter

                        DoubleValidator { id: dv }

                        validator: (new Set([Check.CardInputMethod.Number, Check.CardInputMethod.PaymentCode])).has(currentMethod.cardInputMethod) ? dv : null
                        inputMask: currentMethod.cardInputMethod == Check.CardInputMethod.Phone ? "+7(999)999-99-99" : ""

                        Component.onCompleted: () => forceActiveFocus()
                    }
                }

                Layout.fillWidth: true
            }

            InlineKeyboard {
                layoutType: "digits"
                Layout.preferredWidth: 735
                Layout.preferredHeight: 350
                Layout.alignment: Qt.AlignHCenter
            }
        }

        ColumnLayout {
            spacing: 15

            ActionButton {
                id: okButton
                Style.class: "inputCardConfirm"
                visible: currentMethod.manual
                enabled: inputField.acceptableInput

                Tr { code: "inputCardApply" }

                onClicked: () => root.accept()

                Layout.fillWidth: true
                Layout.preferredHeight: 105
            }

            ActionButton {
                id: cancelButton
                Style.class: "inputCardCancel"

                Tr { code: "inputCardCancel" }

                onClicked: () => root.reject()

                Layout.fillWidth: true
                Layout.preferredHeight: 105
            }

            Layout.fillWidth: true
        }
    }
}
