import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Shapes

import Check
import Core

import "../widgets"

Item {
    id: root
    clip: true

    property bool edit
    property color backgroundColor

    property alias model: list.model
    property alias delegate: list.delegate
    property alias interactive: list.interactive

    function scrollToEnd() { list.positionViewAtEnd() }

    RowLayout {
        spacing: 10
        anchors.fill: parent

        component QuantityChange: Item {
            id: qtyChange
            property int posId
            property var qty
            property string units

            RowLayout {
                anchors.fill: parent
                spacing: 20

                ActionButton {
                    visible: root.edit
                    Style.class: "icon"
                    contentItem: SvgImage {
                        source: qty.fractional || qty.str === "1" ? Style.parameter("icon.position.qtyDelete") : Style.parameter("icon.position.qtyMinus")
                    }
                    padding: 15

                    Layout.preferredWidth: 60
                    Layout.preferredHeight: 60

                    onClicked: () => {
                        Check.selectItemByPosId(qtyChange.posId)
                        PluginManager.async("CHECK_POSITIONQUANTITYMINUS")
                    }
                }

                Label {
                    Style.class: "quantity"
                    text: qty.str + " " + units
                    horizontalAlignment: Qt.AlignHCenter
                    verticalAlignment: Qt.AlignVCenter

                    Layout.preferredWidth: 90
                    Layout.preferredHeight: 60

                    MouseArea {
                        anchors.fill: parent
                        enabled: !qty.fractional

                        onClicked: () => {
                            Check.selectItemByPosId(qtyChange.posId)
                            PluginManager.async("CHECK_CHANGEQUANTITY")
                        }
                    }
                }

                ActionButton {
                    visible: root.edit
                    Style.class: "icon"
                    contentItem: SvgImage {
                        visible: !qty.fractional
                        source: Style.parameter("icon.position.qtyPlus")
                    }
                    padding: 15

                    Layout.preferredWidth: 60
                    Layout.preferredHeight: 60

                    onClicked: () => {
                        Check.selectItemByPosId(qtyChange.posId)
                        PluginManager.async("CHECK_POSITIONQUANTITYPLUS")
                    }
                }
            }
        }

        component Position: Item {
            id: position
            property int posId
            property string name
            property var quantity
            property string units
            property var sum
            property var sumNoDisc
            property var disc
            property int age

            RowLayout {
                anchors.fill: parent
                spacing: 40

                Label {
                    Style.class: "positionName"
                    text: name
                    elide: Label.ElideRight
                    wrapMode: Text.WordWrap
                    maximumLineCount: 2
                    Layout.fillWidth: true
                }

                GroupBox {
                    Style.class: "positionAgeVerify"
                    visible: age > 0
                    topPadding: 7
                    rightPadding: 5
                    bottomPadding: 7
                    leftPadding: 5

                    contentItem: Label {
                        Style.class: "positionAgeVerify"
                        text: "18+"
                    }
                }

                QuantityChange {
                    posId: position.posId
                    qty: position.quantity
                    units: position.units

                    Layout.preferredWidth: 220
                    Layout.preferredHeight: 60
                }

                ColumnLayout {
                    Layout.minimumWidth: 130

                    Money {
                        Style.class: "positionSumNoDisc"
                        visible: sumNoDisc !== sum
                        discount: true
                        backgroundColor: root.backgroundColor
                        money: sumNoDisc

                        Layout.alignment: Qt.AlignRight
                    }

                    Money {
                        Style.class: "positionSum"
                        money: sum

                        Layout.alignment: Qt.AlignRight
                    }
                }
            }
        }

        ListView {
            id: list

            CheckModel { id: checkModel }
            model: FilterModel {
                sourceModel: checkModel
                filterFunction: (pos) => pos.type == CheckTypes.ItemType.Position
            }

            property var newPos: []

            delegate: Control {
                id: itemContainer
                width: list.width
                leftPadding: 20
                rightPadding: 20
                height: 100

                property bool crossingTop: y < list.contentY
                property bool crossingBottom: y + height > list.contentY + list.height
                property bool fullVisible: !crossingTop && !crossingBottom
                opacity: fullVisible ? 1 : Math.pow(1 - (crossingTop ? (list.contentY - y) / height : ((y + height) - (list.contentY + list.height)) / height), 2)

                contentItem: Item {
                    Position {
                        anchors.fill: parent
                        posId: model.posId
                        name: model.name
                        quantity: model.quantity
                        units: model.units
                        sum: model.sum
                        disc: model.disc
                        sumNoDisc: model.sumNoDisc
                        age: model.age
                    }
                }

                background: Rectangle {
                    color: Style.color("green")
                    radius: 20
                    opacity: 0

                    NumberAnimation on opacity {
                        easing.type: Easing.InExpo
                        running: list.newPos.indexOf(model.row) !== -1
                        from: 0.25; to: 0
                        duration: Style.animationDuration("positionList.highlight", 1000)

                        onFinished: () => {
                            const idx = list.newPos.indexOf(model.row)
                            if (idx !== -1) list.newPos.splice(idx, 1)
                        }
                    }
                }

                Rectangle {
                    color: Style.color("graySoft")
                    height: 1
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.bottom: parent.bottom
                }
            }

            Connections {
                target: root.model

                function onRowsInserted(_parent, first, last) {
                    for (let i = first; i <= last; ++i)
                        list.newPos.push(i)
                    Qt.callLater(list.positionViewAtIndex, last, ListView.Contain)
                }
            }

            ScrollBar.vertical: scrollBar

            onHeightChanged: () => {
                if (list.newPos.length > 0)
                    list.positionViewAtIndex(list.newPos[list.newPos.length - 1], ListView.Contain)
            }

            Layout.fillWidth: true
            Layout.fillHeight: true
        }

        ScrollBar {
            id: scrollBar
            policy: ScrollBar.AlwaysOn
            padding: 0

            background: Rectangle {
                color: Style.color("graySoft")
                radius: width / 2
            }

            contentItem: Rectangle {
                color: Style.color("green")
                radius: width / 2
            }

            Behavior on Layout.preferredWidth {
                NumberAnimation { easing.type: Easing.InOutQuad; duration: Style.animationDuration("positionList.scrollBar", 250) }
            }

            Layout.preferredWidth: size < 1 ? 10 : 0
            Layout.fillHeight: true
        }
    }
}
