import QtQml.Base
import QtQuick
import QtQuick.Layouts

import Core
import I18n
import Sco
import Dialog

import "../core"
import "../widgets"

ModalForm {
    id: root
    Style.class: "inputForm"

    property var action: DialogState.actions[Context.id.value]

    onActivated: () => inputField.forceActiveFocus()

    Context.context {
        inputSources: InputSources.inputSources(action.inputSources)
        logoActionInfo: (action.hideInput || hideMarkInputCancel) ? (() => this.logoAction()) : false
    }

    readonly property bool kbdInput: (Context.inputSources & InputSources.Keyboard) != 0

    property bool hideMarkInputCancel: false
    property bool callAttendantOnHideMarkInputCancel: false
    property bool showControls: false

    Component.onCompleted: () => {
        if (action.textType == DialogTypes.TextType.Mark && Sco.customerMode && !action.hideControls) {
            this.hideMarkInputCancel = Config.getBool("Dialog:hideMarkInputCancel")
            this.callAttendantOnHideMarkInputCancel = Config.getBool("Dialog:callAttendantOnMarkInputCancel")
        }
    }

    function logoAction() {
        if (hideMarkInputCancel && !cancelButton.visible) {
            const action = Action.create("AUTH_CALLATTENDANT", { showCancel: true, $parent: this.action.$origin })
            PluginManager.async(action)
        } else {
            root.showControls = true
            image.visible = image.visible && !action.hideControls
        }
    }

    function accept() {
        if (!action.canBeEmpty && (!inputField.acceptableInput || inputField.text.length === 0))
            return
        const setText = !(action.textType === DialogTypes.TextType.Mark && action.canBeEmpty)
        action.text = setText ? inputField.text : ""
        action.canceled = false
        removeContext()
    }

    function reject() {
        if (callAttendantOnHideMarkInputCancel && Sco.customerMode) {
            PluginManager.async("AUTH_CALLATTENDANT", { $parent: this.action.$origin })
            return
        }
        action.text = String()
        action.canceled = true
        removeContext()
    }

    contentItem: ColumnLayout {
        spacing: 40

        Label {
            id: title
            objectName: "title"
            Style.class: "inputFormTitle"
            horizontalAlignment: Label.AlignHCenter
            wrapMode: Label.WordWrap

            Tr { tr: action.title }

            Layout.fillWidth: true
        }

        Label {
            id: message
            objectName: "message"
            Style.class: "inputFormMessage"
            horizontalAlignment: Label.AlignHCenter
            wrapMode: Label.WordWrap

            Tr {
                tr: controls.visible || action.hideControlsMsg.empty
                        ? action.message
                        : action.hideControlsMsg
            }

            Layout.fillWidth: true
        }

        SvgImage {
            id: image
            objectName: "image"
            visible: !kbdInput.visible

            property bool forceBarcode: action.image.type == ImageData.None && action.textType == DialogTypes.TextType.Barcode

            source: forceBarcode ? "hint.barcode" : action.image

            Layout.preferredHeight: forceBarcode ? Style.parameter("geometry.inputBarcodeImageHeight") : action.imageHeight != -1 ? action.imageHeight : implicitHeight
            Layout.alignment: Qt.AlignHCenter
        }

        ColumnLayout {
            id: controls
            objectName: "controls"
            visible: (!root.action.hideControls || root.showControls) && (root.kbdInput || buttonsLayout.show)
            spacing: 40

            ColumnLayout {
                id: kbdInput
                objectName: "kbdInput"
                visible: root.kbdInput
                spacing: 40

                GroupBox {
                    Style.class: "inputFormInputField"

                    contentItem: FocusScope {
                        focus: true

                        implicitHeight: inputField.implicitHeight

                        TextEdit {
                            id: inputField
                            anchors.fill: parent
                            objectName: "inputField"

                            inputMethodHints: Qt.ImhDigitsOnly
                            echoMode: action.hideInput ? TextInput.Password : TextInput.Normal
                            focus: true

                            validator: action.regexp.length > 0 ? Utils.validatorRegularExpression(action.regexp) : null
                            inputMask: action.mask

                            Keys.onEnterPressed: () => root.accept()
                            Keys.onReturnPressed: () => root.accept()
                            Keys.onEscapePressed: () => root.reject()

                            Component.onCompleted: () => {
                                this.text = action.text
                                this.forceActiveFocus()
                            }
                        }
                    }

                    Layout.fillWidth: true
                }

                InlineKeyboard {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 350
                    layoutType: "digits"
                }
            }

            ColumnLayout {
                id: buttonsLayout
                spacing: 15

                property bool showCancel: action.showCancel && (!root.hideMarkInputCancel || action.textType != DialogTypes.TextType.Mark || !Sco.customerMode)
                property bool show: root.kbdInput || showCancel
                visible: show

                ActionButton {
                    id: okButton
                    objectName: "okButton"

                    visible: root.kbdInput || (root.action.canBeEmpty && buttonsLayout.showCancel)
                    enabled: root.action.canBeEmpty || inputField.acceptableInput && inputField.text.length > 0

                    Tr { tr: action.okButton; fallback: "inputOk" }

                    onClicked: () => root.accept()

                    Layout.fillWidth: true
                    Layout.preferredHeight: 105
                }

                ActionButton {
                    id: cancelButton
                    Style.class: "outlined"
                    objectName: "cancelButton"
                    icon: Style.parameter("icon.input.cancel")

                    visible: buttonsLayout.showCancel

                    Tr { tr: action.cancelButton; fallback: "inputCancel" }

                    onClicked: () => root.reject()

                    Layout.fillWidth: true
                    Layout.preferredHeight: 105
                }

                Layout.fillWidth: true
            }

            Layout.fillWidth: true
            Layout.fillHeight: true
        }

        ColumnLayout {
            id: additionalButtons

            property bool showManualBarcode: !controls.visible && action.textType == DialogTypes.TextType.Barcode
            property bool showHelp: !controls.visible && !showManualBarcode
            property bool show: showManualBarcode || showHelp
            visible: show

            ActionButton {
                id: manualBarcode
                Style.class: "inputBarcodeManual"
                icon: Style.parameter("icon.input.manual")

                visible: additionalButtons.showManualBarcode

                Tr { code: "checkManualInputBarcodeManual" }

                onClicked: () => root.showControls = true

                Layout.fillWidth: true
                Layout.preferredHeight: 105
            }

            HelpButton {
                id: help
                Style.class: "inputHelpButton"
                visible: parent.showHelp

                Tr { code: "inputHelpButton" }

                Layout.fillWidth: true
                Layout.preferredHeight: 105
            }

            Layout.fillWidth: true
            Layout.preferredHeight: 105
        }
    }
}
