import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQml.Models
import Qt5Compat.GraphicalEffects

import Check
import Core
import I18n
import PickList as PL

import "../core"
import "../widgets"

Item {
    id: root

    property size grid: Qt.size(4, 2)
    property real cellWidth: treeView.currentItem.width / grid.width
    property real cellHeight: treeView.currentItem.height / grid.height
    property var model: PL.PickList.pageModel

    signal selected(index: var)

    component PickListItem: GroupBox {
        id: itemBox
        Style.class: "pickListItem"

        leftInset: 5
        topInset: 5
        rightInset: 5
        bottomInset: 5
        padding: 30

        contentItem: Item {
            Label {
                anchors {
                    left: parent.left
                    top: parent.top
                    right: parent.right
                }

                Style.class: "pickListItem"
                text: model.name
                wrapMode: Label.WordWrap
                elide: Label.ElideRight
                maximumLineCount: 2
            }

            Label {
                anchors {
                    left: parent.left
                    bottom: parent.bottom
                }

                visible: !model.isGroup && !model.price.zero
                Style.class: "pickListItemPrice"
                text: model.price.str + (PL.PickList.showUnits && model.unit.length > 0 ? "\n/" + model.unit : "")
            }
        }

        background: Rectangle {
            id: itemBoxBackground
            color: itemBox.Style.background.color
            radius: itemBox.Style.border.radius

            border {
                width: itemBox.Style.border.width
                color: itemBox.Style.border.color
            }

            layer {
                enabled: true
                effect: OpacityMask {
                    maskSource: Rectangle {
                        x: itemBoxBackground.x; y: itemBoxBackground.y
                        width: itemBoxBackground.width; height: itemBoxBackground.height
                        radius: itemBoxBackground.radius
                    }
                }
            }

            Behavior on color {
                ColorAnimation {
                    duration: Style.animationDuration("pickList.item", 100)
                }
            }

            SvgImage {
                anchors {
                    top: parent.top; left: parent.left
                    topMargin: 70; leftMargin: 90
                }

                height: 140
                width: 170

                source: model.image
            }
        }
    }

    property Component searchItem: GroupBox {
        Style.class: "pickListSearch"

        property bool showSearch: search.text.length > 0 || search.activeFocus

        contentItem: TextEdit {
            anchors.fill: parent
            id: search
            horizontalAlignment: TextEdit.AlignHCenter
            verticalAlignment: TextEdit.AlignVCenter
            Style.class: "pickListSearch"
            property var filterIndex: root.model.filterIndex(text)

            Connections {
                target: treeView

                function onBeforePop(oldIndex) {
                    if (oldIndex === search.filterIndex)
                        Qt.callLater(() => search.text = '')
                }
            }

            Timer {
                id: filterTimer
                repeat: false
                interval: 500

                onTriggered: () => {
                    search.filterIndex = root.model.filterIndex(search.text)
                    treeView.push(search.filterIndex)
                }
            }

            onTextChanged: () => {
                if (this.text.length > 0)
                    filterTimer.restart()
                else {
                    filterTimer.stop()
                    root.model.reset()
                }
            }
        }

        ActionButton {
            Style.class: "pickListSearch"
            icon: Style.parameter("icon.pickList.search")
            Tr { code: "pickListSearchHint" }
            anchors.fill: parent
            visible: !parent.showSearch
        }

        MouseArea {
            anchors.fill: parent
            preventStealing: true
            enabled: !parent.showSearch

            onClicked: () => search.forceActiveFocus(Qt.MouseFocusReason)
        }
    }

    ColumnLayout {
        anchors.fill: parent
        spacing: 10

        Label {
            id: title
            Style.class: "pickListTitle"
            text: regenerateTitle()
            elide: Label.ElideLeft

            Layout.alignment: Qt.AlignHCenter | Qt.AlignTop
            Layout.bottomMargin: 20

            function regenerateTitle() {
                const titles = []
                const rootIndex = root.model.rootIndex()
                let index = treeView.currentItem.index

                while (index !== rootIndex) {
                    const name = root.model.data(index, PL.PageModel.Name)
                    if (name.length > 0)
                        titles.push(I18n.ui(name));
                    index = root.model.parent(index)
                }
                titles.push(I18n.ui('pickListLayerRoot'))
                titles.reverse()
                return titles.join(' / ')
            }

            Connections {
                target: root.model

                function onDataChanged() {
                    title.text = title.regenerateTitle()
                }
            }

            Connections {
                target: treeView

                function onAfterPush() {
                    title.text = title.regenerateTitle()
                }

                function onAfterPop() {
                    title.text = title.regenerateTitle()
                }
            }

            Connections {
                target: I18n

                function onCurrentLangChanged() {
                    title.text = title.regenerateTitle()
                }
            }
        }

        Loader {
            sourceComponent: root.searchItem

            Layout.fillWidth: true
            Layout.preferredHeight: 80
        }

        TreeView {
            id: treeView
            model: root.model
            clip: true

            Connections {
                target: root.model

                function onModelReset() {
                    while (treeView.depth > 1)
                        treeView.pop()
                }
            }

            Transition {
                id: fadeIn

                NumberAnimation {
                    property: "opacity"; from: 0; to: 0
                    duration: Style.animationDuration("transition.pickList", 250)
                }

                NumberAnimation {
                    property: "opacity"; from: 0; to: 1
                    duration: Style.animationDuration("transition.pickList", 250)
                }
            }

            Transition {
                id: fadeOut

                NumberAnimation {
                    property: "opacity"; from: 1; to: 0
                    duration: Style.animationDuration("transition.pickLsit", 250)
                }

                NumberAnimation {
                    property: "opacity"; from: 0; to: 0
                    duration: Style.animationDuration("transition.pickList", 250)
                }
            }

            pushEnter: fadeIn
            pushExit: fadeOut
            popEnter: fadeIn
            popExit: fadeOut
            replaceEnter: fadeIn
            replaceExit: fadeOut

            delegate: Item {
                id: listItem

                property var model
                property var index
                property alias count: view.count
                property alias currentIndex: view.currentIndex

                SwipeView {
                    focusPolicy: Qt.ClickFocus

                    id: view
                    anchors.fill: parent

                    Repeater {
                        model: PagedModel {
                            pageSize: root.grid.width * root.grid.height
                            source: listItem.model
                            rootIndex: listItem.index
                        }

                        delegate: GridView {
                            id: gridView
                            model: page
                            interactive: false

                            cellWidth: root.cellWidth
                            cellHeight: root.cellHeight

                            add: fadeIn
                            remove: fadeOut

                            delegate: PickListItem {
                                width: gridView.cellWidth
                                height: gridView.cellHeight

                                focusPolicy: Qt.ClickFocus

                                onClicked: () => {
                                    const selected = page.modelIndex(index)
                                    if (model.isGroup)
                                        treeView.push(selected)
                                    else
                                        root.selected(selected)
                                }
                            }
                        }
                    }
                }
            }

            Layout.fillWidth: true
            Layout.fillHeight: true
        }

        RowLayout {
            ActionButton {
                Style.class: "pickListBack"
                icon: !enabled ? Style.parameter("icon.pickList.backDisabled") : Style.parameter("icon.pickList.backActive")
                iconSize: 24
                enabled: treeView.depth > 1

                onClicked: () => treeView.pop()

                Layout.preferredWidth: 100
                Layout.preferredHeight: 56
            }

            Space { Layout.fillWidth: true }

            PageIndicator {
                id: pageIndicator
                opacity: count > 1 ? 1 : 0
                count: treeView.currentItem.count
                currentIndex: treeView.currentItem.currentIndex
                spacing: 10

                delegate: Rectangle {
                    width: 16
                    height: 16
                    color: index === pageIndicator.currentIndex ? Style.color("greenDark") : Style.color("graySoft")
                    radius: 4

                    Behavior on color { ColorAnimation { duration: Style.animationDuration("pickList.pageIndicator", 100) } }
                }

                Behavior on opacity { NumberAnimation { duration: Style.animationDuration("pickList.pageIndicator", 250) } }

                Layout.preferredWidth: implicitWidth
                Layout.preferredHeight: implicitHeight
                Layout.alignment: Qt.AlignHCenter
            }

            Space { Layout.fillWidth: true }

            ActionButton {
                Style.class: "pickListLeft"
                icon: !enabled ? Style.parameter("icon.pickList.leftDisabled") : Style.parameter("icon.pickList.leftActive")
                iconSize: 24
                enabled: treeView.currentItem.currentIndex > 0

                onClicked: () => --treeView.currentItem.currentIndex

                Layout.preferredWidth: 56
                Layout.preferredHeight: 56
            }

            ActionButton {
                Style.class: "pickListRight"
                icon: !enabled ? Style.parameter("icon.pickList.rightDisabled") : Style.parameter("icon.pickList.rightActive")
                iconSize: 24
                enabled: treeView.currentItem.currentIndex < (treeView.currentItem.count - 1)

                onClicked: () => ++treeView.currentItem.currentIndex

                Layout.preferredWidth: 56
                Layout.preferredHeight: 56
            }
        }
    }
}
